* Our company uses a third party program written in Microsoft QuickBASIC.
* The larger numerical values are stored in "Microsoft Binary Format".
* These are four byte representations of the decimal numbers.  
* We want to be able to transfer the values to and from FoxPro database files.
* I could shell out to DOS and run special QuickBASIC programs that used the
* MKSMBF$ and CVSMBF functions to convert the numbers.  However, my 
* programming philosophy said this was inelegant and by taking advantage of
* the low level file functions, I should be able to do this directly 
* in my FoxPro application. 
* 
* I have tried in vain to find a formula to convert the numbers (posting 
* on Compuserve got one response).  I couldn't find the format of this type
* of numerical * representation anywhere after searching through various 
* libraries.  
*
* I finally rolled up my sleeves and figured out how to do the conversion.
* The result is the following two UDF's, MKSMBF and CVSMBF, that mimic the 
* Microsoft QuickBASIC functions MKSMBF$ and CVSMBF, respectively.
* They are being placed in the public domain for anyone to use freely.  
* The following program demonstrates the two UDF's.
*
* Any comments, etc. can be directed to :
*  Evan Jones
*  Rose Technology Group
*  255 Consumers Road
*  Suite 130
*  Willowdale, Ontario, Canada  M2J 1R4
*  CIS 70034,3262
*
CLEAR
mndec = 1000.0000
@ 10,10 SAY "Decimal number to convert " GET mndec 
READ 
mcbyte = MKSMBF(mndec)
@ 11,10 SAY "Four byte representation              " + mcbyte 
mndec2 = CVSMBF(mcbyte)
@12,10 SAY "Original decimal number    " + STR(mndec2,15,4) 
RETURN

PROCEDURE MKSMBF        && Convert from decimal to 4 byte Microsoft Binary format
*
* Date       Author        Description
* 92 Jul 23  Evan Jones    Original FoxPro 2.0 version
*
* Usage:  mcnumber = MKSMBF(expN)
*
* Returns: l_fourbyte - Four byte Microsoft Binary Format representation of number
*
*
* Variables Passed IN
*   decimalnum  - Decimal number to convert
*
* Local Variables
*   l_exp      - Exponent of power of 2
*   l_rem1     - Difference between decimal number and first representation of number
*   l_rem2     - Difference between decimal number and second representation of number
*   l_rem3     - Difference between decimal number and third representation of number
*   l_byte4    - Last byte, representing exponent of 2
*   l_byte3    - Third byte, representing sign and block 
*   l_byte2    - Second byte, representing decimals (256*l_block)
*   l_byte1    - First byte, representing decimals (256*256*l_block)
*   l_block    - Value of block for calculations
*   l_fourbyte - Four byte Microsoft Binary Format representation of number
*
* Note: the number is represented by the following four bytes
*
*      l_byte1,l_byte2,     l_byte3,           l_byte4
*       
*       decimal bytes   sign & first block   exponent byte
*
   PARAMETER decimalnum
   IF NOT TYPE('decimalnum') == 'N' OR decimalnum=0
      l_fourbyte = CHR(0)+CHR(0)+CHR(0)+CHR(0)
    ELSE
      STORE CHR(0) TO l_byte1, l_byte2, l_byte3, l_byte4
      l_exp = INT(LOG(ABS(decimalnum))/LOG(2))
      l_rem1 = ABS(decimalnum) - 2^l_exp
      IF l_rem1 < 0                    && Adjust exponent if too low
         l_exp = l_exp - 1
         l_rem1 = ABS(decimalnum) - 2^l_exp
      ENDIF
      l_byte4 = CHR(129 + l_exp)
      l_block = 2^(7-l_exp)
      l_byte3 = CHR(l_rem1*l_block)
      l_rem2 = ABS(l_rem1 - ASC(l_byte3)/l_block)
      l_byte2 = CHR(INT(l_rem2*l_block*256))
      l_rem3 = l_rem2 - ASC(l_byte2)/l_block/256
      l_byte1 = CHR(INT(ROUND(l_rem3*l_block*256*256,0)))
      l_byte3 = CHR(l_rem1*l_block + IIF(decimalnum<0,128,0)) && Adjust for sign
      l_fourbyte = l_byte1+l_byte2+l_byte3+l_byte4
   ENDIF
   RETURN l_fourbyte

PROCEDURE CVSMBF        && Convert from 4 byte Microsoft Binary format to a decimal number
*
* Date       Author        Description
* 92 Jul 23  Evan Jones    Original FoxPro 2.0 version
*
* Usage:  mnumber = CVSMBF(expC)
*
* Returns: l_decimal - Decimal equivalent of number
*
* Variables Passed IN
*   fourbyte  - Four byte Microsoft Binary Format representation of number
*
* Local Variables
*   l_exp     - Exponent of power of 2
*   l_num1    - Number to add to total for first byte
*   l_num2    - Number to add to total for second byte
*   l_num3    - Number to add to total for third byte
*   l_num4    - Number to add to total for fourth byte (exponent)
*   l_byte4   - Last byte, representing exponent of 2
*   l_byte3   - Third byte, representing sign and block 
*   l_byte2   - Second byte, representing decimals (256*l_block)
*   l_byte1   - First byte, representing decimals (256*256*l_block)
*   l_block   - Value of block for calculations
*   l_decimal - Decimal equivalent of number
*
* Note: the number is represented by the following four bytes
*
*      l_byte1,l_byte2,     l_byte3,           l_byte4
*       
*       decimal bytes   sign & first block   exponent byte
*
   PARAMETER fourbyte
   IF NOT TYPE('fourbyte') == 'C' 
      l_decimal = 0
    ELSE
      l_byte4 = RIGHT(fourbyte,1)
      l_byte3 = SUBSTR(fourbyte,3,1)
      l_sign = IIF(ASC(l_byte3)>127,-1,1)
      l_byte3 = IIF(ASC(l_byte3)>127,CHR(ASC(l_byte3)-128),l_byte3)
      l_byte2 = SUBSTR(fourbyte,2,1)
      l_byte1 = LEFT(fourbyte,1)
      l_exp = ASC(l_byte4) - 129
      l_block = 2^(7-l_exp)
      l_decimal = l_sign*(ASC(l_byte1)/l_block/256/256 + ;
                         ASC(l_byte2)/l_block/256 + ;
                         ASC(l_byte3)/l_block + 2^l_exp)
   ENDIF
   RETURN l_decimal
